'''This module defines the original SimpleParse
grammar.  It uses the generator objects directly
as this is the first grammar being written.'''

from simpleparse.objectgenerator import *

whitespace = Name (value = "ts", repeating = 1, report = 0)
element_token = Name( value = "element_token" )
literal = Name ( value = "literal")
group = Name ( value = "group")
range = Name ( value = "range")
name = Name ( value = "name")


generator = Generator ()


generator.addDefinition(
	"declarationset",
	Name (value = "declaration", repeating = 1),
)



generator.addDefinition (
	"declaration",
	SequentialGroup (
		children = [
			whitespace,
			FirstOfGroup (
				children = [
					Name (value = "unreportedname", ),
					Name (value = "name", ),
				],
			),
			whitespace,
			Literal (value = ":=",),
			whitespace,
			element_token,
			FirstOfGroup (
				children = [
					Name (value = "seq_added_token", ),
					Name (value = "fo_added_token", ),
				],
				optional = 1, repeating = 1,
			),
			whitespace,
		],
	)
)
generator.addDefinition (
	"seq_added_token",
	SequentialGroup (
		children = [
			whitespace,
			Literal(value = ","),
			whitespace,
			element_token,
		],
		repeating = 1,
	)
)
generator.addDefinition (
	"fo_added_token",
	SequentialGroup (
		children = [
			whitespace,
			Literal(value =","),
			whitespace,
			element_token,
		],
		repeating = 1,
	)
)
generator.addDefinition (
	"element_token",
	SequentialGroup (
		children = [
			Name (value = "negpos_indicator", optional = 1),
			whitespace,
			FirstOfGroup (
				children = [
					literal,
					range,
					group,
					name,
				]
			),
			whitespace,
			Name (value = "occurrence_indicator", optional = 1),
		]
	)
)

generator.addDefinition (
	"group",
	SequentialGroup (
		children = [
			Literal (value ="("),
			whitespace,
			element_token,
			FirstOfGroup (
				children= [
					Name (value = "seq_added_token", ),
					Name (value = "fo_added_token", ),
				],
				optional = 1, repeating = 1,
			),
			whitespace,
			Literal (value =")"),
		]
	)
)

generator.addDefinition (
	"negpos_indicator",
	Range (value = "+-" ),
)	
generator.addDefinition (
	"occurrence_indicator",
	Range (value = "+*?" ),
)	
generator.addDefinition (
	"unreportedname",
	SequentialGroup (
		children = [
			Literal (value ="<"),
			whitespace,
			name,
			whitespace,
			Literal (value =">"),
		]
	)
)

generator.addDefinition (
	"name",
	SequentialGroup (
		children = [
			Range(value ='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_'),
			Range(value ='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789', optional= 1, repeating= 1),
		]
	)
)

generator.addDefinition (
	"ts", # ( [ \011-\015]+ / ('#',-'\n'+,'\n')+ )*
	FirstOfGroup (
		children = [
			Range(value =' \011\012\013\014\015', repeating=1),
			Name( value = "comment" ),
		],
		optional = 1, repeating = 1,
	)
)
generator.addDefinition (
	"comment", # ( [ \011-\015]+ / ('#',-'\n'+,'\n')+ )*
	SequentialGroup (
		children = [
			Literal ( value ="#"),
			Literal (value ="\n", negative = 1, repeating = 1),
			Literal (value = "\n",),
		],
	),
)

generator.addDefinition (
	"literal",  # ("'",(CHARNOSNGLQUOTE/ESCAPEDCHAR)*,"'")  /  ('"',(CHARNODBLQUOTE/ESCAPEDCHAR)*,'"')
	FirstOfGroup (
		children = [
			SequentialGroup (
				children = [
					Literal (value ="'"),
					FirstOfGroup (
						children = [
							Name (value = "CHARNOSNGLQUOTE"),
							Name (value = "ESCAPEDCHAR"),
						],
						optional = 1, repeating = 1,
					),
					Literal (value ="'"),
				],
			),
			SequentialGroup (
				children = [
					Literal (value ='"'),
					FirstOfGroup (
						children = [
							Name (value = "CHARNODBLQUOTE"),
							Name (value = "ESCAPEDCHAR"),
						],
						optional = 1, repeating = 1,
					),
					Literal (value ='"'),
				],
			)
		],
	)
)

generator.addDefinition (
	"range",   # '[',CHARBRACE?,CHARDASH?, (CHARRANGE/CHARNOBRACE)*, CHARDASH?,']'
	SequentialGroup (
		children =[
			Literal (value ="["),
			Name (value ="CHARBRACE",optional = 1),
			Name (value ="CHARDASH",optional = 1),
			FirstOfGroup(
				children = [
					Name (value ="CHARRANGE"),
					Name (value ="CHARNOBRACE"),
				],
				optional = 1, repeating = 1,
			),
			Name (value ="CHARDASH",optional = 1),
			Literal (value ="]"),
		],
	)
)
generator.addDefinition (
	"CHARBRACE",   
	Literal (value = "]"),
)
generator.addDefinition (
	"CHARDASH",   
	Literal (value = "-"),
)
generator.addDefinition (
	"CHARRANGE",   # CHARRANGE           :=  CHARNOBRACE, '-', CHARNOBRACE
	SequentialGroup (
		children =[
			Name (value ="CHARNOBRACE"),
			Literal (value ="-"),
			Name (value ="CHARNOBRACE"),
		],
	),
)
generator.addDefinition (
	"CHARNOBRACE",   # CHARRANGE           :=  CHARNOBRACE, '-', CHARNOBRACE
	FirstOfGroup(
		children =[
			Name (value ="ESCAPEDCHAR"),
			Name (value ="CHAR"),
		],
	),
)
generator.addDefinition (
	"CHAR",
	Literal (
		value ="]",
		negative = 1, repeating = 1,
	),
)

generator.addDefinition (
	"ESCAPEDCHAR",   # '[',CHARBRACE?,CHARDASH?, (CHARRANGE/CHARNOBRACE)*, CHARDASH?,']'
	SequentialGroup (
		children =[
			Literal (value ="\\"),
			FirstOfGroup(
				children = [
					Name (value ="SPECIALESCAPEDCHAR"),
					Name (value ="OCTALESCAPEDCHAR"),
				],
			),
		],
	)
)

generator.addDefinition (
	"SPECIALESCAPEDCHAR",
	Range(value ='\\abfnrtv"\''),
)

generator.addDefinition (
	"OCTALESCAPEDCHAR",   # [0-7],[0-7]?,[0-7]?
	SequentialGroup (
		children =[
			Range (value ="01234567"),
			Range (value ="01234567", optional = 1),
			Range (value ="01234567", optional = 1),
		],
	)
)
generator.addDefinition (
	"CHARNODBLQUOTE",
	Range(value ='\\"', negative = 1, repeating = 1),
)
generator.addDefinition (
	"CHARNOSNGLQUOTE",
	Range(value ="\\'", negative = 1, repeating = 1),
)

declaration = r'''declarationset      :=  declaration+
declaration         :=  ts , (unreportedname/name) ,ts,':=',ts, element_token, ( seq_added_token / fo_added_token / or_added_token / and_added_token )*, ts
seq_added_token     :=  (ts,',',ts, element_token)+
fo_added_token      :=  (ts,'/',ts, element_token)+
element_token       :=  negpos_indicator?, ts, (literal/range/group/name),ts, occurence_indicator?
group               :=  '(',ts, element_token, ( seq_added_token / fo_added_token / or_added_token / and_added_token )*, ts, ')'

negpos_indicator    :=  [-+]
occurence_indicator :=  [+*?]
unreportedname      :=  '<', name, '>'
name                :=  [a-zA-Z_],[a-zA-Z0-9_]*
<ts>                :=  ( [ \011-\015]+ / ('#',-'\n'+,'\n')+ )*
literal             :=  ("'",(CHARNOSNGLQUOTE/ESCAPEDCHAR)*,"'")  /  ('"',(CHARNODBLQUOTE/ESCAPEDCHAR)*,'"')


range               :=  '[',CHARBRACE?,CHARDASH?, (CHARRANGE/CHARNOBRACE)*, CHARDASH?,']'
CHARBRACE           :=  ']'
CHARDASH            :=  '-'
CHARRANGE           :=  CHARNOBRACE, '-', CHARNOBRACE
CHARNOBRACE         :=  ESCAPEDCHAR/CHAR
CHAR                :=  -[]]
ESCAPEDCHAR         :=  '\\',( SPECIALESCAPEDCHAR / OCTALESCAPEDCHAR )
SPECIALESCAPEDCHAR  :=  [\\abfnrtv]
OCTALESCAPEDCHAR    :=  [0-7],[0-7]?,[0-7]?
CHARNODBLQUOTE      :=  -[\\"]+
CHARNOSNGLQUOTE     :=  -[\\']+
'''

if __name__ == "__main__":
	import pprint
	parser = generator.buildParser( "declarationset")
	pprint.pprint( parser )
	