import unittest, pprint
from simpleparse.objectgenerator import *
try:
	from TextTools import TextTools
except ImportError:
	from mx.TextTools import TextTools

class ElementTokenTests(unittest.TestCase):
	def doBasicTest(self, instance, testvalue, expected, startPosition=0 ):
		table = tuple(instance.toParser())
		result = TextTools.tag( testvalue, table , startPosition)
		assert result == expected, '''\n\texpected:%s\n\tgot:%s\n'''%( expected, result )
	def testString1( self ):
		self.doBasicTest(
			Literal( value = 'test' ),
			'test',
			(1, [],4),
		)
	def testString2( self ):
		self.doBasicTest(
			Literal( value = 'test', optional =1 ),
			'test',
			(1, [],4),
		)
	def testString3( self ):
		self.doBasicTest(
			Literal( value = 'test', optional =1, negative=1 ),
			'test',
			(1, [],0),
		)
# not what's expected, but is working correctly :(
##	def testString4( self ):
##		self.doBasicTest(
##			Literal( value = 'test', negative=1 ),
##			'test',
##			(0, [],0),
##		)
	def testString5( self ):
		self.doBasicTest(
			Literal( value = 'test', repeating=1),
			'testtest',
			(1, [],8),
		)
	def testString6( self ):
		self.doBasicTest(
			Literal( value = 'test', repeating=1, optional = 1),
			'testtest',
			(1, [],8),
		)
	def testString7( self ):
		self.doBasicTest(
			Literal( value = 'test', repeating=1, optional = 1, negative = 1),
			'testtest',
			(1, [],0),
		)
# not what's expected, but is working correctly :(
##	def testString8( self ):
##		self.doBasicTest(
##			Literal( value = 'test', repeating=1, negative = 1),
##			'testtest',
##			(0, [],0),
##		)
	def testString9( self ):
		self.doBasicTest(
			Literal( value = '\\',),
			'\\',
			(1, [],1),
		)
	def testRange1( self ):
		self.doBasicTest(
			Range( value = 'abc'),
			'aabbcc',
			(1, [],1),
		)
	def testRange2( self ):
		self.doBasicTest(
			Range( value = 'abc', optional=1),
			'aabbcc',
			(1, [],1),
		)
	def testRange3( self ):
		self.doBasicTest(
			Range( value = 'abc', optional=1, repeating=1),
			'aabbcc',
			(1, [],6),
		)
	def testRange4( self ):
		self.doBasicTest(
			Range( value = 'abc', optional=1, repeating=1, negative=1),
			'aabbcc',
			(1, [],0),
		)
	def testRange5( self ):
		self.doBasicTest(
			Range( value = 'abc', optional=1, negative=1),
			'aabbcc',
			(1, [],0),
		)
	def testRange6( self ):
		self.doBasicTest(
			Range( value = 'abc', negative=1),
			'aabbcc',
			(0, [],0),
		)
	def testRange7( self ):
		self.doBasicTest(
			Range( value = 'abc', negative=1, repeating=1),
			'aabbcc',
			(0, [],0),
		)
	def testRange8( self ):
		self.doBasicTest(
			Range( value = 'abc', negative=1, repeating=1),
			'defc',
			(1, [],3),
		)
	def testRange9( self ):
		self.doBasicTest(
			Range( value = 'abc', negative=1),
			'defc',
			(1, [],1),
		)
	def testSequential1( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=0,
			),
			'atest',
			(1, [],5),
		)
	def testSequential2( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=1,
			),
			'atest',
			(0, [],0),
		)
	def testSequential3( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=1, optional=1,
			),
			'atest',
			(1, [],0),
		)
	def testSequential4( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=1, optional=1, repeating=1,
			),
			'sdatest',
			(1, [],2),
		)
	def testSequential5( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				optional=1, repeating=1,
			),
			'atestbtestctest',
			(1, [],15),
		)
	def testSequential6( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				optional=1, 
			),
			'atestbtestctest',
			(1, [],5),
		)
		
	def testSequential7( self ):
		self.doBasicTest(
			SequentialGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				optional=1, 
			),
			'satestbtestctest',
			(1, [],0),
		)
	def testFirstOf1( self ):
		self.doBasicTest(
			FirstOfGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=0,
			),
			'atest',
			(1, [],1),
		)
	def testFirstOf2( self ):
		self.doBasicTest(
			FirstOfGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=0,
			),
			'testa',
			(1, [],4),
		)
	def testFirstOf3( self ):
		self.doBasicTest(
			FirstOfGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=1,
			),
			'testa',
			(0, [],0),
		)
	def testFirstOf4( self ):
		self.doBasicTest(
			FirstOfGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				negative=1, optional=1,
			),
			'testa',
			(1, [],0),
		)
	def testFirstOf5( self ):
		self.doBasicTest(
			FirstOfGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				repeating=1,
			),
			'testabtest',
			(1, [],10),
		)
	def testFirstOf6( self ):
		self.doBasicTest(
			FirstOfGroup(
				children = [
					Range( value = 'abc',),
					Literal( value = 'test', ),
				],
				repeating=1, negative = 1,
			),
			'hellotheretestabtest',
			(1, [],10),
		)
	def testFirstOf7( self ):
		'''("test"?/"hello")*
		This test exposes a problem with both the original generator
		and the sub-class here.  FOGroups with optional children are
		in danger of never returning as the children always "succeed"
		even if they consume nothing.
		Failure in this case is likely to be an endless loop, so we
		can expect that if this is broken there will be heck to pay ;)
		'''
		import pprint
		generator =	FirstOfGroup(
				children = [
					Range( value = 'abc', optional=1),
					Literal( value = 'test', optional=1),
				],
				repeating=1, optional=1,
			)
		pprint.pprint(
			generator.toParser()
		)
		self.doBasicTest(
			generator,
			'testabtest',
			(1, [],10),
		)
		generator =	FirstOfGroup(
				children = [
					Range( value = 'abc', optional=1),
					Literal( value = 'test', optional=1),
					SequentialGroup(
						children = [
							Literal( value = 'm', optional=1),
							Literal( value = 'n', optional=1),
						],
					),
				],
				repeating=1, optional=1,
			)
		pprint.pprint(
			generator.toParser()
		)
		self.doBasicTest(
			generator,
			'testmnabtest',
			(1, [],10),
		)
		
		
def getSuite():
	return unittest.makeSuite(ElementTokenTests,'test')

if __name__ == "__main__":
	unittest.main(defaultTest="getSuite")
