declaration = r'''
set       := (interesting/multset/plusset)+
multset   := '*',(set/atom), (set/atom)
plusset   := '+',(set/atom), (set/atom)
atom      := -[+*]

interesting := (example8/example7/example6/example5/example4/example3/example2/example1)
example1     := '*+',(set/atom),(set/atom),'+',(set/atom),(set/atom)
example2     := '**',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
example3     := 'fsd*',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
example4     := 'm*',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
example5     := 'a*',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
example6     := 's*',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
example7     := 'bdf*',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
example8     := 'sd*',(set/atom),(set/atom),'++',(set/atom),(set/atom),(set/atom)
'''
import sys, string
from simpleparse import generator
try:
	from TextTools import TextTools
except ImportError:
	from mx.TextTools import TextTools

class Emitter:
	def process( self, data ):
		#import pprint
		tree = self.parse( data )
		#pprint.pprint( tree )
		# wrap up the tuple 'cause TextTools uses a different format for the top-level :(
		tree = ('set',0, tree[-1], tree[1] )
		return self.emit( tree )
	def parse( self, data ):
		self.data = data
		parser = generator.buildParser( declaration ).parserbyname( 'set' )
		return TextTools.tag( data, parser )
	def write( self, data ):
		sys.stdout.write( data )
	def emit( self, tree ):
		'''
		return transformation for a single tuple...
		'''
		if hasattr( self, 'emit' + tree[0] ): # have explicitprocessing function
			func = getattr( self, 'emit'+tree[0] )
			return func( tree )
		else:
			if tree[3]: # children to process, things to do :)
				result = []
				### write out pre-elements
				endpos = tree[3][0][1] # start of first child
				result.append( self.data[ tree[1]:endpos] )
				### write children
				for child in tree[3]:
					result.append( self.emit( child ) )
				### write out post elements
				startpos = tree[3][-1][2] # end of last child
				result.append( self.data[ startpos: tree[2]] )
				return string.join( result, '' )
			else:
				# we're just re-emitting same text...
				return self.data[ tree[1]:tree[2]]
	def emitexample1( self, tuple ):
		'''*+AB+CD -> ++*AC*AD+*BC*BD'''
		#print 'interesting'
		#import pdb
		#pdb.set_trace()
		a,b,c,d = map( self.emit, tuple[3] )
		#print `(a,b,c,d)`,
		return '++*%s%s*%s%s+*%s%s*%s%s'%( a,c,a,d,b,c,b,d)

testdata = [
'''++m*++mkp+f*nkf''',
'''*+ab+cd''',
'''+ab+bc+de''',
'''*ab*bc*de''',
'''++m*++mkp+f*nkf'''*10000,
]

a = Emitter()
import time, profile
for test in testdata:
	t = time.time()
	a.parse( test )
	print 'total time', time.time()-t

#profile.run( '''a.process( testdata[4] )''')
